#include "display_joystick.h"
#include <stdbool.h>

static char board[9];
static const int numWinPatterns = 8;
static const char winPattern[][3] = {{0,1,2}, {3,4,5}, {6,7,8}, {0,3,6}, {1,4,7}, {2,5,8}, {0,4,8}, {2,4,6}};

void initGame(void)
{
  for(int i = 0; i < 9; i++)
  {
    board[i] = '.';
  }
}

static void makeMove(void)
{
  // Versuche, ein Gewinnmuster aus zwei X zu vervollstndigen
  int missingPosition = -1;
  for(int n = 0; n < numWinPatterns; n++)
  {
    int numSet = 0;
    for(int k = 0; k < 3; k++)
    {
      int z = winPattern[n][k];
      if(board[z] == 'x')
      {
        numSet++;
      }
      else
      {
        missingPosition = z;
      }
    }
    if(numSet == 2 && board[missingPosition] == '.') // zwei x richtig, die dritte Stelle frei
    {
      board[missingPosition] = 'x';
      return; //*** Vorsicht: verstecktes return ***
    }
  }
  
  // Kein Gewinnmuster gefunden. Dann setzen wir das x auf den erstmglichen Platz.
  for(int i = 0; i < 9; i++)
  {
    if(board[i] == '.')
    {
      board[i] = 'x';
      return; //*** Vorsicht: verstecktes return ***
    }
  }
}

static bool checkForWin(char c)
{  
  for(int n = 0; n < numWinPatterns; n++)
  {
    bool allSet = true;
    for(int k = 0; k < 3; k++)
    {
      int z = winPattern[n][k];
      if(board[z] != c)
      {
        allSet = false;
        break;
      }
    }
    if(allSet)
    {
      return true;
    }
  }
  return false;
}

static void displayAndWait()
{
  char s[4];
  s[3] = 0;
  int j = 0;
  while(readAnalog(1) > 200)
  {
    if(j == 3)
    {
      writeString("--------");
    }
    else
    {
      for(int i = 0; i < 3; i++)
      {
        s[i] = board[3*j + i];
      }
      writeString(s);
    }
    waitMilliseconds(300);
    j = (j + 1) % 4;
  }
  while(readAnalog(1) < 400)
  {
  }
}

static int getNumber(void)
{
  int z = 4;
  while(readAnalog(1) > 200) // wurde nicht nach unten gedrckt
  {
    int increment = (readAnalog(0) - 512) / 200;
    z += increment;
    if(z > 8)
    {
      z = 8;
    }
    else if(z < 0)
    {
      z = 0;
    }       
    writeNumber(z);
    waitMilliseconds(100); 
  }
  while(readAnalog(1) < 400) // wurde noch nicht wieder losgelassen
  {}
  return z;  
}

static bool validateAndStoreMove(int z)
{
  if(board[z] == '.')
  {
    board[z] = 'o';
    return true;
  }
  else
  {
    return false;
  }
}

void playGame(void)
{
  initGame();
  bool haveWinner = false;
  for(int moveCount = 0; moveCount < 9; moveCount++)
  {
    if(moveCount % 2 == 0)
    {
      makeMove();
      displayAndWait();
      if(checkForWin('x'))
      {
        writeString("Ich gew.");
        haveWinner = true;
        break; //*** Vorsicht: verstecktes break ***
      }
    }
    else
    {   
      bool isMoveAllowed = false;
      while(!isMoveAllowed)
      {
        int z = getNumber();
        isMoveAllowed = validateAndStoreMove(z);
        if(!isMoveAllowed)
        {
          writeString("verboten");
          waitMilliseconds(300);
        }
      }
      if(checkForWin('o'))
      {
        writeString("Du gew.");
        haveWinner = true;
        break; //*** Vorsicht: verstecktes break ***
      }
    } 
  }
  if(!haveWinner)
  {
    writeString("Unent.");
  }
  waitMilliseconds(500);
  displayAndWait();
}
